"""Parsing data from text."""

import string
import base64
import vobject
import datetime
import logging
import registry
from dateutil import relativedelta
from dateutil import rrule
from dateutil import parser
from dateutil import tz
from datastructures import *


#todo -
#  odd but minor dependence on dateutil's default keyword values of 0
#  errorhandling is the buggy part
#  no leapseconds -- dateutil lib doesn't support leap seconds. an error will be thrown.
#  stringToDate ignores tzinfos
#  NEED COMPLETE stringToPeriod()
#  check out timezone stuff for everything

## Parsing values of contentlines

#  see below for parsing entire contentlines

utc = tz.tzutc()

def stringToRecurrence(s): #TODO: need to convert values into datetime, like with "until" below
    return Recurrence(s)

def isDuration(s):
    s = string.upper(s)
    return (string.find(s, "P") != -1) and (string.find(s, "P") < 2)

def stringToPeriod(s, tzinfo=None):
    values   = string.split(s, "/")
    start = stringToDateTime(values[0], tzinfo)
    valEnd   = values[1]
    if isDuration(valEnd): #period-start = date-time "/" dur-value
        dur=stringToDurations(valEnd)[0]
        return (start, dur.timedelta * dur.sign)
    else:
        return (start, stringToDateTime(valEnd, tzinfo) - start)

def stringToBinary(s):
    return base64.decodestring(s)

def stringToBoolean(s):
    if s.upper() == "TRUE":
        return True
    elif s.upper() == "FALSE":
        return False
    else:
        raise "Error: Boolean is neither true nor false: " + str(s)

def stringToURI(s):
    """Returns a string URI."""
    return s

def stringToCalAddress(s):
    """Returns a URI (stored in a string)."""
    return stringToURI(s)

def stringToFloat(s):
    return float(s)

def stringToInteger(s):
    return int(s)

def stringToDate(s, tzinfos=None):
    if tzinfos != None: print "Didn't expect a tzinfos here"
    year  = int( s[0:4] )
    month = int( s[4:6] )
    day   = int( s[6:8] )
    return datetime.date(year,month,day)

def stringToDateTime(s, tzinfo=None):
    """Returns datetime.datetime object."""
    try:
        year   = int( s[0:4] )
        month  = int( s[4:6] )
        day    = int( s[6:8] )
        hour   = int( s[9:11] )
        minute = int( s[11:13] )
        second = int( s[13:15] )
        if len(s) > 15:
            if s[15] == 'Z':
                tzinfo = utc
    except:
        raise vobject.ParseError("%s is not a valid DATE-TIME" % s)
    return datetime.datetime(year, month, day, hour, minute, second, 0, tzinfo)

##    if tzinfo: return parser.parse(s).replace(tzinfo=tzinfo)
##    else: return parser.parse(s)


def stringToUTCOffset(s):
    sign = None
    if s[0] == "-": sign = -1
    else: sign = -1
    hour,minute,second = 0,0,0
    if len(s) == 7:
        hour   = int( s[1:3] )
        minute = int( s[3:5] )
        second = int( s[5:7] )
        totalSeconds = sign * (3600*hour + 60*minute + second)
        return tz.tzoffset(None, totalSeconds)
    else:
        hour   = int( s[1:3] )
        minute = int( s[3:5] )
        totalSeconds = sign * (3600*hour + 60*minute)
        return tz.tzoffset(None, totalSeconds)


def stringToText(s, strict=False):
    return string.join(stringToTextValues(s, strict), ",")


escapableCharList = "\\;,Nn"

def stringToTextValues(s, strict=False):
    """Returns list of strings."""

    def escapableChar (c):
        return c in escapableCharList

    def error(msg):
        if strict:
            raise ParseError(msg)
        else:
            #logger.error(msg)
            print msg

    #vars which control state machine
    charIterator = enumerate(s)
    state        = "read normal"

    current = ""
    results = []

    while True:
        try:
            charIndex, char = charIterator.next()
        except:
            #charIndex += 1
            char = "eof"

        #print (char + " <- " + state)
        #logger.debug(char + " <- " + state)
        if state == "read normal":
            if char == '\\':
                state = "read escaped char"
            elif char == ',':
                state = "read normal"
                results.append(current)
                current = ""
            elif char == "eof":
                state = "end"
            else:
                state = "read normal"
                current = current + char

        elif state == "read escaped char":
            if escapableChar(char):
                state = "read normal"
                if char in 'nN': 
                    current = current + '\n'
                else:
                    current = current + char
            else:
                state = "read normal"
                current = current + char #this is an error, but whatever

        elif state == "end":    #an end state
            if current != "" or len(results) == 0:
                results.append(current)
            return results

        elif state == "error":  #an end state
            return results

        else:
            state = "error"
            error("error: unknown state: '%s' reached in %s" % (state, s))

#print stringToTextValues("blah,slkdj")
#print stringToTextValues(r"blah\,\nslkdj")[0]
# print stringToUTCOffset("-0500")
# print stringToDateTime("19970630T235959Z")
# print "PARSE" + str(parser.parse("P15DT5H0M20S"))
# print datetime.timedelta(days=-1, seconds=0, microseconds=0, milliseconds=0, minutes=0, hours=-2, weeks=0)

def stringToDurations(s, strict=False):
    """Returns list of datastructures.Duration objects."""

    def makeTimedelta(sign, week, day, hour, minute, sec):
        if sign == "-": sign = -1
        else: sign = 1
        week      = int(week)
        day       = int(day)
        hour      = int(hour)
        minute    = int(minute)
        sec       = int(sec)
        return sign * datetime.timedelta(weeks=week, days=day, hours=hour, minutes=minute, seconds=sec)

    def error(msg):
        if strict:
            raise vobject.ParseError(msg)
        else:
            raise vobject.ParseError(msg)
            #logger.error(msg)
    
    #vars which control state machine
    charIterator = enumerate(s)
    state        = "start"

    durations = []
    current   = ""
    sign      = None
    week      = 0
    day       = 0
    hour      = 0
    minute    = 0
    sec       = 0

    while True:
        try:
            charIndex, char = charIterator.next()
        except:
            charIndex += 1
            char = "eof"

        #print (char + " <- " + state)
        #logger.debug(char + " <- " + state)
        if state == "start":
            if char == '+':
                state = "start"
                sign = char
            elif char == '-':
                state = "start"
                sign = char
            elif char.upper() == 'P':
                state = "read field"
            elif char == "eof":
                state = "error"
                error("got end-of-line while reading in duration: " + s)
            elif char in string.digits:
                state = "read field"
                current = current + char   #update this part when updating "read field"
            else:
                state = "error"
                print "got unexpected character %s reading in duration: %s" % (char, s)
                error("got unexpected character %s reading in duration: %s" % (char, s))

        elif state == "read field":
            if (char in string.digits):
                state = "read field"
                current = current + char   #update part above when updating "read field"   
            elif char.upper() == 'T':
                state = "read field"
            elif char.upper() == 'W':
                state = "read field"
                week    = current
                current = ""
            elif char.upper() == 'D':
                state = "read field"
                day     = current
                current = ""
            elif char.upper() == 'H':
                state = "read field"
                hour    = current
                current = ""
            elif char.upper() == 'M':
                state = "read field"
                minute  = current
                current = ""
            elif char.upper() == 'S':
                state = "read field"
                sec     = current
                current = ""
            elif char == ",":
                state = "start"
                durations.append( makeTimedelta(sign, week, day, hour, minute, sec) )
                current   = ""
                sign      = None
                week      = None
                day       = None
                hour      = None
                minute    = None
                sec       = None  
            elif char == "eof":
                state = "end"
            else:
                state = "error"
                error("got unexpected character reading in duration: " + s)
            
        elif state == "end":    #an end state
            #print "stuff: %s, durations: %s" % ([current, sign, week, day, hour, minute, sec], durations)

            if (sign or week or day or hour or minute or sec):
                durations.append( makeTimedelta(sign, week, day, hour, minute, sec) )
            return durations

        elif state == "error":  #an end state
            error("in error state")
            return durations

        else:
            state = "error"
            error("error: unknown state: '%s' reached in %s" % (state, line))

# print datetime.timedelta(-10, -2, -11, -22, -33)
# print -1 * datetime.timedelta(10, 2, 11, 22, 33)
# print datetime.timedelta(10, 2, 11, 22, 33)
# print "digits: " + string.digits
# for i in stringToDurations ("P15DT5H0M20S"): print "result of stringToDurations:" + str(i)
# for i in stringToDurations ("-P15DT5H0M20S"): print "result of stringToDurations:" + str(i)
# print stringToDurations ("P15DT5H0M20S")
#print base64.decodestring("MIICajCCAdOgAwIBAgICBEUwDQYJKoZIhvcNAQEEBQAwdzELMAkGA1UEBhMCVVMxLDAqBgNVBAoTI05ldHNjYXBlIENvbW11bmljYXRpb25zIENvcnBvcmF0aW9uMRwwGgYDVQQLExNJbmZv")


## Parsing contentlines

def parseProdid(contentline):
    return stringToText(contentline.value)[0]

def parseVersion(contentline):
    value = stringToText(contentline.value)[0]
    if ";" in value:
        return string.split(value, ";", 2)
    return value

def parseCalscale(contentline):
    return stringToText(contentline.value)[0]

def parseMethod(contentline):
    return stringToText(contentline.value)[0]
        
def parseTzid(contentline):
    return stringToText(contentline.value)[0]
        
def parseLastmod(contentline):
    return stringToDateTime(contentline.value)
        
def parseTzurl(contentline):
    return stringToURI(contentline.value)

def parseDtstart(contentline):
    from registry import getTzid
    tzinfo = getTzid(contentline.params.get("TZID", [None])[0])
    valueParam = contentline.params.get("VALUE", ["DATE-TIME"])[0]
    if valueParam.upper() == "DATE":
        return stringToDate(contentline.value)
    elif valueParam.upper() == "DATE-TIME":
        return stringToDateTime(contentline.value, tzinfo)

def parseRrule(contentline):
    return stringToRecurrence(contentline.value)

def getFromDict(dict, key, default=None):
    answer = default
    if dict.has_key(key):
        return dict[key]
    return answer

def parseText (contentline):
    return Text(stringToTextValues(contentline.value),
                getFromDict(contentline.params, "ALTREP"),
                getFromDict(contentline.params, "LANGUAGE"))

def parseDateTime(contentline):
    return stringToDateTime(contentline.value)

def parseInteger(contentline):
    return stringToInteger(contentline.value)

def parseURI (contentline):
    return stringToURI(contentline.value)

def parseUTCOffset (contentline):
    return stringToUTCOffset(contentline.value)

def parseRecur (contentline, dtstart=None):
    return rrule.rrulestr(str(contentline.value), dtstart=dtstart)

def isDurationCorrupt(text):
    tCount = 0
    fail = True
    for i in text:
        if i in string.digits:
            pass
        elif i.upper() == "T":
            tCount = tCount + 1
            if tCount > 1:
                fail = False
                break
        else:
            fail = False
            break
    return fail

def parseDuration (contentline):
    if isDurationCorrupt(contentline.value):
        return stringToDateTime(contentline.value)
    return stringToDurations(contentline.value)[0]

def parseUnknown (contentline):
    raise "unsupported"

def parseCalAddress (contentline):
    raise "unsupported"

def parseBeginEnd (contentline):
    return contentline.value

def parseOrganizer (contentline):
    return Organizer(stringToCalAddress(contentline.value),
                     {"CN":getFromDict(contentline.params, "CN"),
                      "DIR":getFromDict(contentline.params, "DIR"),
                      "SENT-BY":getFromDict(contentline.params, "SENT-BY"),
                      "LANGUAGE":getFromDict(contentline.params, "LANGUAGE")})

def parseFreeBusy (contentline):
    periods = map(stringToPeriod, string.split(contentline.value, ","))
    fbtype  = getFromDict(contentline.params, "FBTYPE")
    return FreeBusy(periods, type)

#missing is partstatparam
def parseAttendee (contentline):
    return Attendee(stringToCalAddress(contentline.value),
                    {"CUTYPE":getFromDict(contentline.params, "CUTYPE"),
                     "MEMBER":getFromDict(contentline.params, "MEMBER"),
                     "ROLE":getFromDict(contentline.params, "ROLE"),
                     "RSVP":getFromDict(contentline.params, "RSVP"),
                     "DELEGATED-TO":getFromDict(contentline.params, "DELEGATED-TO"),
                     "DELEGATED-FROM":getFromDict(contentline.params, "DELEGATED-FROM"),
                     "CN":getFromDict(contentline.params, "CN"),
                     "DIR":getFromDict(contentline.params, "DIR"),
                     "SENT-BY":getFromDict(contentline.params, "SENT-BY"),
                     "LANGUAGE":getFromDict(contentline.params, "LANGUAGE")})

# this duplicate was found above.
        
# def parseRdate(contentline):
#     value = getFromDict(contentline.params, "value", "DATE-TIME")
#     tzid  = getFromDict(contentline.params, "tzid", None)
#     if contentline.params["value"] == "DATE-TIME":
#         return stringToDateTime(contentline.value, tzinfos=tzid)
#     elif contentline.params["value"] == "DATE":
#         return stringToDate(contentline.value)
#     else:
#         return stringToPeriod(contentline.value)

def parseRdate(line):
    """Parse an RDATE ContentLine.
    
    @return: A list of datetimes, dates, or a (datetime, timedelta) tuple for
        periods.
    
    """
    tzinfo = registry.getTzid(line.params.get("TZID", ["UTC"])[0])
    valueParam = line.params.get("VALUE", ["DATE-TIME"])[0]
    valTexts = string.split(line.value, ",")
    if valueParam.upper() == "DATE":
        return map(stringToDate, valTexts)
    elif valueParam.upper() == "DATE-TIME":
        return [stringToDateTime(x, tzinfo) for x in valTexts]
    elif valueParam.upper() == "PERIOD":
        return [stringToPeriod(x, tzinfo) for x in valTexts]

#print stringToPeriod("19970101T180000Z/19970102T070000Z")
#print parseRdate(vobject.textLineToContentLine("RDATE;VALUE=DATE:19970304,19970504,19970704,19970904"))
#print parseRdate(vobject.textLineToContentLine("RDATE;VALUE=PERIOD:19960403T020000Z/19960403T040000Z,19960404T010000Z/PT3H"))


#def parseRdate (contentline):
#    tzidParam  = getFromDict(contentline.params, "TZID")
#    valueParam = getFromDict(contentline.params, "VALUE", ["DATE-TIME"])
#    valTexts   = string.split(contentline.value, ",")
#    if (valueParam != None) and (valueParam[0].upper() == "DATE"):
#        return DateRdate( map(stringToDate, valTexts), tzidParam )
#    elif (valueParam != None) and (valueParam[0].upper() == "PERIOD"):
#        return PeriodRdate( map(stringToPeriod, valTexts), tzidParam )
#    else:
#        return DateTimeRdate( map(stringToDateTime, valTexts), tzidParam )

def parseTrigger (contentline):
    valueParam = getFromDict(contentline.params, "VALUE", ["DURATION"])
    if valueParam[0].upper() == "DATE-TIME":
        return stringToDateTime(contentline.value)
    else:
        duration = None
        if isDurationCorrupt(contentline.value):
            duration = stringToDateTime(contentline.value)
        else:
            duration = stringToDurations(contentline.value)[0]
        related = getFromDict(contentline.params, "RELATED")
        return Trigger(duration,related)

def parseAttach (contentline):
    encodingParam = getFromDict(contentline.params, "ENCODING")
    valueParam    = getFromDict(contentline.params, "VALUE")
    if (valueParam != None) and (valueParam[0].upper() == "BINARY"):
        return stringToBinary(contentline.value)
    else:
        formatParam = getFromDict(contentline.params, "FMTTYPE")
        return Attachment(formatParam, stringToURI(contentline.value))

def parseGeo (contentline):
    floats = map(lambda x: stringToFloat(x), string.split(contentline.value, ";", 1))
    return GeoPosition(floats[0], floats[1])

def deUnicodeComponent(contentline):
    #name = contentline.name.encode() #let's not encode name
    value = contentline.value.encode()
    params = []
    for key in contentline.params:
        val = contentline.params[key]
        tmp = [key.encode()]
        tmp.extend(map(lambda x: x.encode(), val))
        params.append(tmp)
    newContentline = vobject.ContentLine(contentline.name, params, value)
    return newContentline

def deUnicode(fnOfOneArg):
    return lambda x: fnOfOneArg(deUnicodeComponent(x))

# Table which determines what procedures are used to parse contentline,
# depending on header.
headerToParser = {"calscale"        : parseText,
                  "method"          : parseText,
                  "prodid"          : parseText,
                  "version"         : parseText,
                  "attach"          : parseAttach,
                  "categories"      : parseText,
                  "class"           : parseText,
                  "comment"         : parseText,
                  "description"     : parseText,
                  "geo"             : parseGeo,
                  "location"        : parseText,
                  "percent-complete": parseInteger,
                  "priority"        : parseInteger,
                  "resources"       : parseText,
                  "status"          : parseText,
                  "summary"         : parseText,
                  "completed"       : parseDateTime,
                  "dtend"           : deUnicode(parseDtstart),
                  "due"             : deUnicode(parseDtstart),
                  "dtstart"         : deUnicode(parseDtstart),
                  "duration"        : deUnicode(parseDuration),
                  "freebusy"        : deUnicode(parseFreeBusy),
                  "transp"          : parseText,
                  "tzid"            : parseText,
                  "tzname"          : parseText,
                  "tzoffsetfrom"    : parseUTCOffset,
                  "tzoffsetto"      : parseUTCOffset,
                  "tzurl"           : parseURI,
                  "attendee"        : parseAttendee,
                  "contact"         : parseText,
                  "organizer"       : parseOrganizer,
                  "recurrence-id"   : deUnicode(parseDtstart),
                  "related-to"      : parseText,
                  "url"             : parseURI,
                  "uid"             : parseText,
                  "exdate"          : deUnicode(parseRdate),
                  "exrule"          : parseRecur,
                  "rdate"           : deUnicode(parseRdate),
                  "rrule"           : parseRecur,
                  "action"          : parseText,
                  "repeat"          : parseInteger,
                  "trigger"         : deUnicode(parseTrigger),
                  "created"         : deUnicode(parseDateTime),
                  "dtstamp"         : deUnicode(parseDateTime),
                  "last-modified"   : deUnicode(parseDateTime),
                  "sequence"        : parseInteger,
                  "unknown"         : parseText,
                  "request-status"  : parseText,
                  "begin"           : parseBeginEnd,
                  "end"             : parseBeginEnd
                  }

def parse (contentline):
    """Take a ContentLine instance, which is just a datastructure
    holding text, and turn it into useful data which a user might
    conveniently operate on.

    For example, something with a datetime might be parsed into some
    data holding date/timezone/etc objects.
    """
    parserFn = headerToParser[contentline.name.lower()]
    return parserFn(contentline)
