"""Long or boring tests for vobjects."""

import vobject, behavior, registry

#------------------- Testing and running functions -----------------------------
def _test():
    import doctest, vobject, tests, icalendar, __init__, re
    doctest.testmod(vobject, verbose=0)
    doctest.testmod(tests, verbose=0)
    doctest.testmod(icalendar, verbose=0)
    doctest.testmod(__init__, verbose=0)
    
if __name__ == '__main__':
    _test()
    
#------------------------------ Tests ------------------------------------------
# See http://cvs.sourceforge.net/viewcvs.py/freeassociation/libical/test-data/
# for good sampledata.


testSilly="""
sillyname:name
profile:sillyprofile
stuff:folded
 line
""" + "morestuff;asinine:this line is not folded, \
but in practice probably ought to be, as it is exceptionally long, \
and moreover demonstratively stupid"

icaltest=r"""BEGIN:VCALENDAR
CALSCALE:GREGORIAN
X-WR-TIMEZONE;VALUE=TEXT:US/Pacific
METHOD:PUBLISH
PRODID:-//Apple Computer\, Inc//iCal 1.0//EN
X-WR-CALNAME;VALUE=TEXT:Example
VERSION:2.0
BEGIN:VEVENT
SEQUENCE:5
DTSTART;TZID=US/Pacific:20021028T140000
DTSTAMP:20021028T011706Z
SUMMARY:Coffee with Jason
UID:EC9439B1-FF65-11D6-9973-003065F99D04
DTEND;TZID=US/Pacific:20021028T150000
BEGIN:VALARM
TRIGGER;VALUE=DURATION:-P1D
ACTION:DISPLAY
DESCRIPTION:Event reminder\, with comma\nand line feed
END:VALARM
END:VEVENT
BEGIN:VTIMEZONE
TZID:US/Pacific
LAST-MODIFIED:19870101T000000Z
BEGIN:STANDARD
DTSTART:19671029T020000
RRULE:FREQ=YEARLY;BYDAY=-1SU;BYMONTH=10
TZOFFSETFROM:-0700
TZOFFSETTO:-0800
TZNAME:PST
END:STANDARD
BEGIN:DAYLIGHT
DTSTART:19870405T020000
RRULE:FREQ=YEARLY;BYDAY=1SU;BYMONTH=4
TZOFFSETFROM:-0800
TZOFFSETTO:-0700
TZNAME:PDT
END:DAYLIGHT
END:VTIMEZONE
END:VCALENDAR"""

badstream = r"""BEGIN:VCALENDAR
CALSCALE:GREGORIAN
X-WR-TIMEZONE;VALUE=TEXT:US/Pacific
METHOD:PUBLISH
PRODID:-//Apple Computer\, Inc//iCal 1.0//EN
X-WR-CALNAME;VALUE=TEXT:Example
VERSION:2.0
BEGIN:VEVENT
DTSTART:20021028T140000Z
BEGIN:VALARM
TRIGGER:20021028T120000Z
ACTION:DISPLAY
DESCRIPTION:This trigger is a date-time without a VALUE=DATE-TIME parameter
END:VALARM
END:VEVENT
END:VCALENDAR"""

__test__ = { "Test readComponents" :
    """
    >>> import StringIO
    >>> from vobject import readComponents
    >>> f2 = StringIO.StringIO(testSilly) 
    >>> silly = readComponents(f2).next()
    >>> silly
    <sillyprofile| [<MORESTUFF{}this line is not folded, but in practice probably ought to be, as it is exceptionally long, and moreover demonstratively stupid>, <SILLYNAME{}name>, <STUFF{}foldedline>]>
    >>> silly.stuff[0]
    <STUFF{}foldedline>
    >>> original = silly.serialize()
    >>> f3 = StringIO.StringIO(original)
    >>> silly2 = readComponents(f3).next()
    >>> silly2.serialize()==original
    True
    >>> s3 = StringIO.StringIO('cn:Babs Jensen\\r\\ncn:Barbara J Jensen\\r\\nsn:Jensen\\r\\nemail:babs@umich.edu\\r\\nphone:+1 313 747-4454\\r\\nx-id:1234567890\\r\\n')
    >>> ex1 = readComponents(s3).next()
    >>> ex1
    <*unnamed*| [<CN{}Babs Jensen>, <CN{}Barbara J Jensen>, <EMAIL{}babs@umich.edu>, <PHONE{}+1 313 747-4454>, <SN{}Jensen>, <X-ID{}1234567890>]>
    >>> ex1.serialize()
    u'CN:Babs Jensen\\r\\nCN:Barbara J Jensen\\r\\nEMAIL:babs@umich.edu\\r\\nPHONE:+1 313 747-4454\\r\\nSN:Jensen\\r\\nX-ID:1234567890\\r\\n'
    """,
    
    "Validate sampledata.txt" :
    """
    >>> import vobject
    >>> f=open("sampledata.txt")
    >>> c=list(vobject.readComponents(f, validate=True))
    >>> c[1].vevent[0].dtend[0].value
    datetime.datetime(1998, 3, 12, 9, 30, tzinfo=<tzicalvtz 'US-Eastern'>)
    >>> c[1].vevent[0].dtend[0].value.astimezone(registry.getTzid("UTC"))
    datetime.datetime(1998, 3, 12, 14, 30, tzinfo=tzutc())
    """,
    
    "Import icaltest" :
    """
    >>> import vobject, StringIO, __init__
    >>> f = StringIO.StringIO(icaltest)
    >>> __init__.readTZ(StringIO.StringIO(icaltest))
    >>> c = vobject.readComponents(f, validate=True).next()
    >>> c.vevent[0].valarm[0].trigger[0]
    <TRIGGER{}-1 day, 0:00:00>
    >>> c.vevent[0].rruleset[0]
    datetime.datetime(2002, 10, 28, 14, 0, tzinfo=<tzicalvtz 'US/Pacific'>)
    >>> c.vevent[0].dtend[0].value
    datetime.datetime(2002, 10, 28, 15, 0, tzinfo=<tzicalvtz 'US/Pacific'>)
    >>> c.vevent[0].dtstamp[0].value
    datetime.datetime(2002, 10, 28, 1, 17, 6, tzinfo=tzutc())
    >>> c.vevent[0].valarm[0].description[0].value
    u'Event reminder, with comma\\nand line feed'
    >>> c.vevent[0].valarm[0].description[0].serialize()
    u'DESCRIPTION:Event reminder\\\\, with comma\\\\nand line feed\\r\\n'
    >>> f.seek(0) #read icaltest in a different way
    >>> ev =__init__.readICalendar(f).next()
    >>> ev.rruleset[0] == c.vevent[0].rruleset[0]
    True
    >>> print ev.fragmentString[:12]
    BEGIN:VEVENT
    >>> f.seek(0) #read icaltest's top content lines
    >>> top = __init__.readICalendar(f, topLevel = True).next()
    >>> top.calscale[0]
    <CALSCALE{}GREGORIAN>
    """,
    
    "Parsing tests" :
    """
    >>> import parsing
    >>> parsing.stringToTextValues('')
    ['']
    >>> parsing.stringToTextValues('abcd,efgh')
    ['abcd', 'efgh']
    """,
    
    "Read just TZ" :
    """
    >>> import __init__, StringIO, registry
    >>> __init__.readTZ(StringIO.StringIO(icaltest))
    >>> registry.getTzid('US/Pacific')
    <tzicalvtz 'US/Pacific'>
    """,
    
    "read failure" :
    """
    >>> import StringIO, __init__
    >>> f = StringIO.StringIO(badstream)
    >>> vevent = __init__.readICalendar(f).next()
    Traceback (most recent call last):
    ...
    ParseError: In component starting at line 15: got unexpected character reading in duration: 20021028T120000Z
    """,
    
    "unicode test" :
    """
    >>> import StringIO, __init__
    >>> f = file('utf8_test.ics', 'rb')
    >>> vevent = __init__.readICalendar(f).next()
    >>> vevent.summary[0].value
    u'The title \\u3053\\u3093\\u306b\\u3061\\u306f\\u30ad\\u30c6\\u30a3'
    >>> summary = vevent.summary[0].value
    """,
    
    "regular expression test" :
    """
    >>> import re, vobject
    >>> re.findall(vobject.patterns['name'], '12foo-bar:yay')
    ['12foo-bar', 'yay']
    >>> re.findall(vobject.patterns['safe_char'], 'a;b"*,cd')
    ['a', 'b', '*', 'c', 'd']
    >>> re.findall(vobject.patterns['qsafe_char'], 'a;b"*,cd')
    ['a', ';', 'b', '*', ',', 'c', 'd']
    >>> re.findall(vobject.patterns['param_value'], '"quoted";not-quoted;start"after-illegal-quote', re.VERBOSE)
    ['"quoted"', '', 'not-quoted', '', 'start', '', 'after-illegal-quote', '']
    >>> match = vobject.line_re.match('TEST;ALTREP="http://www.wiz.org":value:;"')
    >>> match.group('value')
    'value:;"'
    >>> match.group('name')
    'TEST'
    >>> match.group('params')
    ';ALTREP="http://www.wiz.org"'
    """
    }
